package main

/**
 *
 * 运行前：请先填写AppId、AppKey, 并根据实际情况填写请求参数
 *
 * 实时语音翻译调用demo
 * 此demo只是一个简单的调用示例，不适合用到实际生产环境中
 *
 */

import (
	"bufio"
	"encoding/json"
	"fmt"
	"github.com/gorilla/websocket"
	"go/main/util"
	"log"
	"os"
)

func main() {
	// connect to host
	dialer := websocket.Dialer{}
	connect, _, err := dialer.Dial(WssAsrUrl, nil)
	if nil != err {
		log.Println(err)
		return
	}

	defer connect.Close()

	// send data
	go sendData(connect)

	// listener: receive
	var receiveTtsAudioBytes []byte
	for {
		messageType, messageData, err := connect.ReadMessage()
		if nil != err {
			log.Println(err)
			break
		}
		switch messageType {
		case websocket.TextMessage:     // 文本数据
			fmt.Println(string(messageData))
		case websocket.BinaryMessage:   // 二进制数据
			receiveTtsAudioBytes = append(receiveTtsAudioBytes, messageData[1:]...)
		default:
		}
	}

	// 保存返回的tts播报数据
	if len(receiveTtsAudioBytes) != 0 {
		err := util.WriteBytes(receiveTtsAudioBytes, TtsAudioSavePath)
		if err != nil {
			log.Println(err)
		}
	}
}

func sendData(connect *websocket.Conn) {
	err := sendStartFrame(connect)
	if err != nil {
		log.Println(err)
		return
	}

	err1 := sendAudioData(connect)
	if err1 != nil {
		log.Println(err1)
		return
	}

	err2 := sendFinishFrame(connect)
	if err2 != nil {
		log.Println(err2)
		return
	}
}

func sendStartFrame(connect *websocket.Conn) error {
	startFrame := getStartFrame()
	result, _ := json.Marshal(&startFrame)
	err := connect.WriteMessage(websocket.TextMessage, result)
	return err
}

func sendAudioData(connect *websocket.Conn) error {
	f, err := os.Open(AudioFilePath)
	if err != nil {
		return err
	}
	defer f.Close()

	var chunkLength = AudioRate * 2 / 1000 * int(AudioSliceLength)
	reader := bufio.NewReader(f)
	buf := make([]byte, chunkLength)

	for {
		_, err = reader.Read(buf)
		if err != nil {
			return err
		}

		err = connect.WriteMessage(websocket.BinaryMessage, buf)
		if err != nil {
			return err
		}
	}
}

func sendFinishFrame(connect *websocket.Conn) error {
	finishFrame := getFinishFrame()
	result, _ := json.Marshal(&finishFrame)
	err := connect.WriteMessage(websocket.TextMessage, result)
	return err
}